/********************************
 *
 * Example to compute the energy function
 * based on Ptools
 *
 *
 * Authors: Sébastien Verel, Sébastien Fiorucci
 * 
 * Date:
 *   version 1 - 5/11/2013
 *
 *******************************/

// in-out of c++
#include <iostream>

// include to have solution representation
#include <minimizers/solution/solution.h>

// includes to compute the energy function
#include <attractrigidbody.h>
#include <minimizers/solution/eval.h>

/*
 * The main function
 */
int main(int argc, char** argv) {
    
  //-----------------------------------------------
  // Fitness function (energy function)
  
  // read the receptor and the ligand
  PTools::AttractRigidbody receptor(argv[1]);
  PTools::AttractRigidbody ligand(argv[2]);

  // constructor with: parameters of the field, file with the receptor, file with the ligand
  Eval eval("../data/parmw.par", receptor, ligand);

  //-----------------------------------------------
  // Example of the best solution

  std::cout << " ************** The optimal position " << std::endl;

  // the solution: vector of 6 real numbers
  // coord[0], coord[1], coord[2] : 3 angles of the position
  // coord[3], coord[4], coord[5] : (x, y, z)

  // the solution
  Solution solution;
  
  // compute the energy of the solution
  eval.apply(solution);

  // print only the fitness (energy)
  std::cout << solution.fitness() << std::endl;

  // print the solution on the output
  solution.printOn();
  std::cout << std::endl;

  //-----------------------------------------------
  // Example by "hand"

  std::cout << " ************** The solution by hand " << std::endl;

  solution.coord[0] = 0.1; // theta
  solution.coord[1] = 0.5; // phi
  solution.coord[2] = 1.6; // psy
  solution.coord[3] = 50;  // x
  solution.coord[4] = 70;  // y
  solution.coord[5] = 100; // z

  // compute the energy of the solution
  eval.apply(solution);

  // print only the fitness
  std::cout << solution.fitness() << std::endl;

  // print on the output
  solution.printOn();
  std::cout << std::endl;

}
