/********************************
 *
 * Example of random initialization
 * based on Ptools
 *
 *
 * Authors: Sébastien Verel, Sébastien Fiorucci
 * 
 * Date:
 *   version 1 - 5/11/2013
 *
 *******************************/

// c++ in-out 
#include <iostream>
// random numbers 
#include <random>

// include to have solution representation
#include <minimizers/solution/solution.h>

// includes to compute the energy function
#include <attractrigidbody.h>
#include <minimizers/solution/eval.h>

// initialization
#include <minimizers/init/fixedDistanceInit.h>

// to save the molecule
#include <pdbio.h>

/*
 * The main function
 *
 * Arguments:
 *   1 : receptor file (required)
 *   2 : ligand file   (required)
 *   3 : random seed 
 */
int main(int argc, char** argv) {
  // random generator
  std::mt19937 rng;
  
  if (argc > 3)
    // the random seed is given to be able to re-run the experiment
    rng.seed(atoi(argv[3]));


  //-----------------------------------------------
  // Fitness function (energy function)

  // read the receptor and the ligand
  PTools::AttractRigidbody receptor(argv[1]);
  PTools::AttractRigidbody ligand(argv[2]);

  // constructor with: parameters of the field, file with the receptor, file with the ligand
  Eval eval("../data/parmw.par", receptor, ligand);
  
  //-----------------------------------------------
  // Example of random initialization of the position on a sphere

  // objet of initialization (see minimizers/init/fixedDistanceInit.h)
  FixedDistanceInit init(rng, receptor, ligand);

  std::cout << " ************** Random position " << std::endl;

  // the solution
  Solution solution;
  
  // apply the intialisation operator on the solution
  init.apply(solution);

  // compute the energy of the solution
  eval.apply(solution);

  // print the solution on the output
  solution.printOn();
  std::cout << std::endl;

  //-----------------------------------------------
  // save into file the ligand at the random position
  
  /*
  // save the solution into a file
  std::fstream file("test.sol");
  
  if (file) {
    solution.printOn(file);
    
    file.close();
  } else {
    std::cerr << "Error file not found: " << "test.sol" << std::endl;
  }
  */
  
  /*
  // save the molecule

  // move to this random solution
  solution.move(ligand);

  // write the the ligand into pdb format file
  PTools::WritePDB(ligand, "ligandRnd.red2");
  */
}
