/********************************
 *
 * Example of (empty) minimizer
 * based on Ptools
 *
 *
 * Authors: 
 * 
 * Date:
 *   version 1 - 5/11/2013
 *
 *******************************/

// in-out of c++
#include <iostream>
// random numbers 
#include <random>

// include to have solution representation
#include <minimizers/solution/solution.h>

// includes to compute the energy function
#include <attractrigidbody.h>
#include <minimizers/solution/eval.h>

// example of minimizer class
#include <minimizers/algo/es.h>

/*
 * The main function
 */
int main(int argc, char** argv) {
  // random generator
  std::mt19937 rng;
  
  if (argc >= 3)
    // the random seed is given to be able to re-run the experiment
    rng.seed(atoi(argv[3]));

  //-----------------------------------------------
  // Fitness function (energy function)

  // read the receptor and the ligand
  PTools::AttractRigidbody receptor(argv[1]);
  PTools::AttractRigidbody ligand(argv[2]);

  // constructor with: parameters of the field, file with the receptor, file with the ligand
  Eval eval("../data/parmw.par", receptor, ligand);
  
  //-----------------------------------------------
  // Example of minimization

  EvolutionStrategy minimizer(rng, eval);

  minimizer.run();

  // the final (best) solution
  Solution solution = minimizer.final();
  
  // print the solution on the output
  solution.printOn();
  std::cout << std::endl;

}
